"""
Herman Wijaya
Student ID: 1003975
SUTD - MSSD (2018)

et-classifier.py
"""

import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split, StratifiedKFold
from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score, confusion_matrix
from sklearn.preprocessing import RobustScaler
from sklearn.ensemble import ExtraTreesClassifier
import time


def convert(dataset, lookup):
    X = []
    for i in range(len(dataset)//lookup):
        data = [dataset[i*lookup:(i+1)*lookup, 1:6]]
        X.append(data)
    #for i in range(len(dataset)-lookup):
    #    data = [dataset[i:i+lookup, 1:6]]
    #    X.append(data)
    return np.array(X)

def kFoldCrossVal(k, model, X, y, scaling=False):
    # K-Fold cross-validation
    kfold = StratifiedKFold(n_splits=k, shuffle=True)
    acc_scores = []
    prec_scores = []
    rec_scores = []
    f1_scores = []
    for train, test in kfold.split(X, y):
        X_train, X_test = X[train], X[test]
        y_train, y_test = y[train], y[test]

        # Scaler
        if scaling == True:
            scaler = RobustScaler()
            scaler.fit(X_train)
            X_train = scaler.transform(X_train)
            X_test = scaler.transform(X_test)

        # Model Training
        model.fit(X_train, y_train)

        # Testing
        y_pred = model.predict(X_test)
        acc_scores.append(accuracy_score(y_test, y_pred))
        prec_scores.append(precision_score(y_test, y_pred))
        rec_scores.append(recall_score(y_test, y_pred))
        f1_scores.append(f1_score(y_test, y_pred))

    np.set_printoptions(formatter={'float': '{:.5f}'.format})
    print(f"K-Fold cross-validation scores")
    print(f"------------------------------")
    print(f"Accuracy:\n{np.array(acc_scores)}")
    print(f"Precision:\n{np.array(prec_scores)}")
    print(f"Recall:\n{np.array(rec_scores)}")
    print(f"F1:\n{np.array(f1_scores)}")

# Data extraction from CSVs
normal_df = pd.read_csv('data/swat-normal.csv', index_col=False)
abnormal_df = pd.read_csv('data/swat-abnormal.csv', index_col=False)

lookup = 90

normal = convert(normal_df.values, lookup)
abnormal = convert(abnormal_df.values, lookup)
normal_X, normal_y = normal, np.array([0] * len(normal))
abnormal_X, abnormal_y = abnormal, np.array([1] * len(abnormal))

X = np.concatenate((normal_X, abnormal_X))
X = X.reshape(X.shape[0], -1)
y = np.concatenate((normal_y, abnormal_y))

# Result reproducibility
np.random.seed(1)

# Split Train-Test data 0.6:0.4
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.4)

print("=====ExtraTrees=====")

# Model training
model = ExtraTreesClassifier(n_estimators=100, criterion='gini', bootstrap=False, max_features='auto', verbose=0)
start = time.time()
model.fit(X_train, y_train)
stop = time.time()
print(f"Time to build model: {stop-start:.5f}s")

# Testing
y_pred = model.predict(X_test)
print(f"Accuracy: {accuracy_score(y_test, y_pred):.5f}")
print(f"Precision: {precision_score(y_test, y_pred):.5f}")
print(f"Recall: {recall_score(y_test, y_pred):.5f}")
print(f"F1: {f1_score(y_test, y_pred):.5f}")

# Confusion matrix
con_mtx = confusion_matrix(y_test, y_pred).ravel() # (tn, fp, fn, tp)
print(f"[TN, FP, FN, TP]: {con_mtx}\n")

# K-Fold cross-validation
kFoldCrossVal(5, model, X, y)
