"""
Herman Wijaya
Student ID: 1003975
SUTD - MSSD (2018)

if-outlier.py
"""

import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split, cross_val_score
from sklearn.metrics import accuracy_score, confusion_matrix
from sklearn.ensemble import IsolationForest
import time

def convert(dataset, lookup):
    X = []
    for i in range(len(dataset)//lookup):
        data = [dataset[i*lookup:(i+1)*lookup, 1:6]]
        X.append(data)
    #for i in range(len(dataset)-lookup):
    #    data = [dataset[i:i+lookup, 1:6]]
    #    X.append(data)
    return np.array(X)

# Data extraction from CSVs
normal_df = pd.read_csv('data/swat-normal.csv', index_col=False)
abnormal_df = pd.read_csv('data/swat-abnormal.csv', index_col=False)

lookup = 90

normal = convert(normal_df.values, lookup)
abnormal = convert(abnormal_df.values, lookup)
normal_X, normal_y = normal, np.array([1] * len(normal))
abnormal_X, abnormal_y = abnormal, np.array([- 1] * len(abnormal))

#X = normal_X
X = np.concatenate((normal_X, abnormal_X))
X = X.reshape(X.shape[0], -1)
#y = normal_y
y = np.concatenate((normal_y, abnormal_y))

# Result reproducibility
np.random.seed(1)

# Split Train-Test data 0.6:0.4
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.4)

# Model training with normal data only
model = IsolationForest(n_estimators=100, behaviour='new', contamination=0.36, bootstrap=False, verbose=0)
start = time.time()
print(model.fit_predict(X_train))
stop = time.time()
print(f"Time to build model: {stop-start:.5f}s")
print(f"Training accuracy: {accuracy_score(y_train, model.predict(X_train)):.5f}")

# Testing with both normal and abnormal data
#abnormal_X = abnormal_X.reshape(abnormal_X.shape[0], -1)
#X_test = np.concatenate((X_test, abnormal_X))
#y_test = np.concatenate((y_test, abnormal_y))
y_pred = model.predict(X_test)
print(y_pred)

# Confusion matrix
y_true = y_test
con_mtx = confusion_matrix(y_true, y_pred).ravel() # (tp, fn, fp, tn)
print(f"Accuracy: {accuracy_score(y_test, y_pred):.5f}")
prec = con_mtx[0]/(con_mtx[0]+con_mtx[2])
print(f"Precision: {prec:.5f}")
rec = con_mtx[0]/(con_mtx[0]+con_mtx[1])
print(f"Recall: {rec:.5f}")
f1 = 2*prec*rec/(prec+rec)
print(f"F1: {f1:.5f}")
print(f"[TP, FN, FP, TN]: {con_mtx}")
